import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:equatable/equatable.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/models/device_status.dart';
import 'package:syncrow_web/services/devices_mang_api.dart';

part 'living_room_event.dart';
part 'living_room_state.dart';

class LivingRoomBloc extends Bloc<LivingRoomEvent, LivingRoomState> {
  LivingRoomBloc() : super(LivingRoomInitial()) {
    on<LivingRoomFetchDeviceStatus>(_onFetchDeviceStatus);
    on<LivingRoomControl>(_livingRoomControl);
  }

  FutureOr<void> _onFetchDeviceStatus(
      LivingRoomFetchDeviceStatus event, Emitter<LivingRoomState> emit) async {
    emit(LivingRoomDeviceStatusLoading());
    try {
      final status =
          await DevicesManagementApi().getDeviceStatus(event.deviceId);
      emit(LivingRoomDeviceStatusLoaded(status));
    } catch (e) {
      emit(LivingRoomDeviceManagementError(e.toString()));
    }
  }

  FutureOr<void> _livingRoomControl(
      LivingRoomControl event, Emitter<LivingRoomState> emit) async {
    emit(LivingRoomControlLoading());
    try {
      final status = Status(code: event.code, value: event.value);
      final response =
          await DevicesManagementApi().deviceControl(event.deviceId, status);

      if (response) {
        await Future.delayed(const Duration(milliseconds: 500));
        final newStatus =
            await DevicesManagementApi().getDeviceStatus(event.deviceId);
        emit(LivingRoomDeviceStatusLoaded(newStatus));
      } else {
        emit(const LivingRoomControlError('Failed to control the device.'));
      }
    } catch (e) {
      emit(LivingRoomControlError('Error controlling the device: $e'));
    }
  }
}
